/**
 * This function (getNextHop) makes a query to the CO-BGP table (Actually it uses an intermediate
 * "table" (a text file called "COtable.txt", which is populated by the monitoring script. See
 * www.netlab.tkk.fi/~vholopai/stuff.shtml for details), and returns the next hop.
 *
 * This code can be freely used for any purposes, NO WARRANTY!
 *
 * author: Visa Holopainen (visa.holopainen@aalto.fi)
 **/

#include "hashtbl.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <sys/types.h>
#include <time.h>

int getNextHop(const char *num, char *nextIP)
{
	FILE *fp, *fp2;
	char string[30], seps[] = "->\n";
	char *token, number[30], nexthop[30], *nxt, *subnum;
	HASHTBL *hashtbl;
	int nrof_routes = 0, i;

	/* -------------------------------------- first populate the hash table --------------------------------- */
	fp2 = fopen("./nrof_routes.txt", "r");
	if (!fp2)
	{
		fprintf(stderr, "You must have the CO monitor running with subroutine write_plaintext! It will populate the file nrof_routes.txt.\n");
		return -1;
	}
	fscanf(fp2,"%d",&nrof_routes);
	fclose(fp2);

	/* fprintf(stderr, "nrof routes: %d\n", nrof_routes); */

	if(!(hashtbl=hashtbl_create(nrof_routes, NULL)))
	{
		fprintf(stderr, "ERROR: hashtbl_create() failed\n");
		return -1;
	}

	fp = fopen("./COtable.txt", "r");
	if (!fp)
	{
		fprintf(stderr, "You must have the CO monitor running with subroutine write_plaintext! It will populate the file COtable.txt.\n");
		return -1;
	}
	while(fgets( string, sizeof(string)-1, fp) != NULL)
	{
		/* Break into tokens. */
		token = string;
		if ( token[0] != '-' )
		{
			token = strtok( string, seps );
			sprintf(number, "%s", token);
			token = strtok( NULL, seps );
			sprintf(nexthop, "%s", token);
			/* fprintf(stderr, "insert %s -> %s\n", number, nexthop); */
			hashtbl_insert(hashtbl, number, nexthop);
		}
		else /* default CO route */
		{
			token = strtok( string, seps );
			sprintf(nexthop, "%s", token);
			/* fprintf(stderr, "insert default -> %s\n", nexthop); */
			hashtbl_insert(hashtbl, "default", nexthop);
		}
	}
	fclose(fp);

	/* -------------------------------------- then make the longest prefix lookup  --------------------------------- */
	for ( i=0; i<13; i++ )
	{
		/* fprintf(stderr, "%d\n", i); */
		subnum = strndup(num, 13-i);
		nxt = hashtbl_get(hashtbl, subnum);
		if ( nxt ){
			strcpy(nextIP, nxt);
			break;
		}
	}
	nxt = hashtbl_get(hashtbl, "default");
	if ( nxt ){
		/* fprintf(stderr, "got default -> %s\n", nxt); */
		strcpy(nextIP, nxt);
	}
	return 0;
}
